@@ -161,9 +161,10 @@ void    resolve_clnt_init(RESOLVE_REPLY *reply)
 void    resolve_clnt(const char *class, const char *sender,
 		             const char *addr, RESOLVE_REPLY *reply)
 {
-    char   *myname = "resolve_clnt";
+    const char *myname = "resolve_clnt";
     VSTREAM *stream;
     int     server_flags;
+    int     count = 0;
 
     /*
      * One-entry cache.
@@ -226,6 +227,7 @@ void    resolve_clnt(const char *class, const char *sender,
     for (;;) {
 	stream = clnt_stream_access(rewrite_clnt_stream);
 	errno = 0;
+	count += 1;
 	if (attr_print(stream, ATTR_FLAG_NONE,
 		       ATTR_TYPE_STR, MAIL_ATTR_REQ, class,
 		       ATTR_TYPE_STR, MAIL_ATTR_SENDER, sender,
@@ -239,7 +241,7 @@ void    resolve_clnt(const char *class, const char *sender,
 			 ATTR_TYPE_STR, MAIL_ATTR_RECIP, reply->recipient,
 			 ATTR_TYPE_INT, MAIL_ATTR_FLAGS, &reply->flags,
 			 ATTR_TYPE_END) != 5) {
-	    if (msg_verbose || (errno != EPIPE && errno != ENOENT))
+	    if (msg_verbose || count > 1 || (errno && errno != EPIPE && errno != ENOENT))
 		msg_warn("problem talking to service %s: %m",
 			 var_rewrite_service);
 	} else {
